program TestSprite;

{ http://shinh.skr.jp/testsprite/testsprite.adb ̃x^ڐA }

{$APPTYPE CONSOLE}

{$IFDEF FPC}{$MODE DELPHI}{$ENDIF}

uses
	SDL;

const
	 DefaultNumSprites = 100;
	 MaxSpeed = 1;
	 ScreenWidth = 640; { 640 800 1024 }
	 ScreenHeight =  480; { 480 600  768 }

var
	Screen: PSDL_Surface;
	Sprite: PSDL_Surface;
	SpritesVisible: Boolean;
	SpriteRects, Positions, Velocities: array of SDL_Rect;
	numsprites: Cardinal;

function LoadSprite(Screen : PSDL_Surface; const FileName : string): Boolean;
var
	Temp : PSDL_Surface;
begin
	{ Load the sprite image }
	Sprite := SDL_LoadBmp(PChar(FileName));
	if Sprite = nil then
	begin
		WriteLn('Couldn''t load ', FileName, ' : ', SDL_GetError);
		Result := False;
	end
	else
	begin
		{ Set transparent pixel as the pixel at (0,0) }
		if Sprite^.format^.palette <> nil then
		begin
			SDL_SetColorKey(Sprite,	(SDL_SRCCOLORKEY or SDL_RLEACCEL),
				PUint32(Sprite.pixels)^);
		end;
		{ Convert sprite to video format }
		Temp := SDL_DisplayFormat(Sprite);
		if Temp = nil then
		begin
			WriteLn('Couldn''t convert background: ', SDL_GetError);
			Result := False;
		end
		else
		begin
			Sprite := Temp;
			{ we're ready to roll. :) }
			Result := True;
		end;
	end;
end;

procedure MoveSprites(Screen: PSDL_Surface; background : Uint32);
var
	nupdates: Cardinal;
	Area: SDL_Rect;
	Position, Velocity: PSDL_Rect;
  I: Integer;
begin
	nupdates := 0;
	{ Erase all sprites if necessary }
	if SpritesVisible then
		SDL_FillRect(screen, nil, background);
	{ Move the sprite, bounce at the wall, and draw }
	for i := Low(SpriteRects) to High(SpriteRects) do
	begin
		Position := @Positions[I];
		Velocity := @Velocities[I];
		Position^.x := position^.x + velocity^.x;
		if (position^.x < 0) or (position^.x >= screen.w - 32) then
		begin
			velocity.x := -velocity.x;
			Position.x := position.x + velocity.x;
		end;
		position.y := position.y + velocity.y;
		if (position^.y < 0) or (position^.y >= screen.h - 32) then
		begin
			velocity.y := -velocity.y;
			Position.y := position.y + velocity.y;
		end;
		{ Blit the sprite onto the screen }
		area := position^;
		SDL_BlitSurface(sprite, nil, screen, @Area);
		SpriteRects[nupdates] := area;
		nupdates := nupdates + 1;
	end;
	{ Update the screen }
	if screen.flags and SDL_DOUBLEBUF = SDL_DOUBLEBUF then
		SDL_Flip(screen)
	else
	begin
		SDL_Flip(screen);
		{ V.UpdateRects(screen, C.int(nupdates), sprite_rects.all); }
	end;
	SpritesVisible := True;
end;

function FastestFlags(flags: UINT32): UINT32;
var
	info: PSDL_VideoInfo;
begin
	Result := Flags;
	{ Hardware aceleration is only used in fullscreen mode }
	Result := Result or SDL_FULLSCREEN;
	{ Check for various video capabilities }
	info := SDL_GetVideoInfo;
	{SDL.pasɒ萔`̂ōĂ邩s}
	if (info^.blit_hw and 2 <> 0) and (info^.blit_hw and 128 <> 0) then
	begin
		{ We use accelerated colorkeying and color filling }
		Result := Result or SDL_HWSURFACE;
	end;
  { If we have enough video memory, and will use accelerated 
		blits directly to it, then use page flipping. }
	if Result and SDL_HWSURFACE = SDL_HWSURFACE then
	begin
		{ Direct hardware blitting without double-buffering
			causes really bad flickering. }
		if info.video_mem > ScreenWidth * ScreenHeight then
			Result := Result or SDL_DOUBLEBUF
		else
			Result := Result and not SDL_HWSURFACE;
	end;
end;

var
	VideoBPP: Uint8;
	videoflags: UINT32;
	background: Uint32;
	done: Integer;
	event: TSDL_Event;
	AndThen, now, frames: Uint32;
	argc: 0 .. High(Integer);
	I, Value: Integer;
	Code: Integer; {Val's error position}
	DST: SDL_Rect;
begin
	Argc := ParamCount;
	if SDL_Init(SDL_INIT_VIDEO) < 0 then
	begin
		WriteLn('Couldn''t initialize SDL: ', SDL_GetError);
		ExitCode := 1;
	end
	else
	begin
		try
			numsprites := DefaultNumSprites;
			videoflags := SDL_SWSURFACE or SDL_ANYFORMAT;
			VideoBPP := 8;
			WriteLn('Numero de argumentos: ', argc);
			while argc > 0 do
			begin
				Val(ParamStr(argc), Value, Code); {Test and Convert}
				if (argc > 1) and (ParamStr(argc - 1) = '-bpp') and (Code = 0) then
				begin
					VideoBPP := Value;
					videoflags := videoflags and not SDL_ANYFORMAT;
					argc := argc - 2;
					WriteLn('-bpp', VideoBPP);
				end
				else if ParamStr(argc) = '-fast' then
				begin
					videoflags := FastestFlags(videoflags);
					argc := argc - 1;
					WriteLn('-fast');
				end
				else if ParamStr(argc) = '-hw' then
				begin
					videoflags := videoflags xor SDL_DOUBLEBUF;
					argc := argc -1;
					WriteLn('-hw');
				end
				else if ParamStr(argc) = '-fullscreen' then
				begin
					videoflags := videoflags xor SDL_FULLSCREEN;
					argc := argc - 1;
					WriteLn('-fullscreen');
				end
				else if Code = 0 then
				begin
					numsprites := Value;
					argc := argc - 1;
					WriteLn('numsprites: ', numsprites);
				end
				else
				begin
					 WriteLn('Usage: ', ParamStr(0), #$20,
						'[-bpp N] [-hw] [-flip] [-fast]',
						'[-fullscreen] [numsprites]');
					 ExitCode := 1;
					 Exit;
				end;
			end;
			SetLength(SpriteRects, numsprites);
			SetLength(positions, numsprites);
			SetLength(velocities, numsprites);
			{ Set video mode }
			screen := SDL_SetVideoMode(ScreenWidth, ScreenHeight, VideoBPP, videoflags);
			if screen = nil then
			begin
				WriteLn('Couldn''t set ', ScreenWidth, 'x', ScreenHeight,
					' video mode: ', SDL_GetError);
				ExitCode := 1;
				Exit;
			end;
			{ Load the sprite }
			if not LoadSprite(screen, 'icon.bmp') then
			begin
				ExitCode := 1;
				Exit;
			end;
			Randomize;
			for I := 0 to NumSprites do
			begin
				positions[i].x := Random(ScreenWidth);
				positions[i].y := Random(ScreenHeight);
				positions[i].w := sprite^.w;
				positions[i].h := sprite^.h;
				velocities[i].x := 0;
				velocities[i].y := 0;
				while (velocities[i].x = 0) and (velocities[i].y = 0) do
				begin
					 velocities[i].x := Random(2 * MaxSpeed) - MaxSpeed;
					 velocities[i].y := Random(2 * MaxSpeed) - MaxSpeed;
				end;
			end;
			background := SDL_MapRGB(screen.format, $00, $00, $00);
			{ Print out information about our surfaces }
			WriteLn('Screen is at ', screen.format.BitsPerPixel, ' bits per pixel');
			if screen.flags and SDL_HWSURFACE = SDL_HWSURFACE then
				WriteLn('Screen is in video memory')
			else
				WriteLn('Screen is in system memory');
			if screen.flags and SDL_DOUBLEBUF = SDL_DOUBLEBUF then
				WriteLn('Screen has double-buffering enabled');
			if sprite.flags and SDL_HWSURFACE = SDL_HWSURFACE then
				WriteLn('Sprite is in video memory')
			else
				WriteLn('Sprite in in system memory');
			{ Run a sample blit to trigger blit aceleration }
			dst.x := 0;
			dst.y := 0;
			dst.w := sprite.w;
			dst.h := sprite.h;
			SDL_BlitSurface(sprite, nil, screen, @dst);
			SDL_FillRect(screen, @dst, background);
			if sprite.flags and SDL_HWACCEL = SDL_HWACCEL then
				WriteLn('Sprite blit uses hardware acceleration');
			if sprite.flags and SDL_RLEACCEL = SDL_RLEACCEL then
				WriteLn('Sprite blit uses RLE acceleration');
			{ Loop, blitting sprite and waiting for a keystroke }
			frames := 0;
			AndThen := SDL_GetTicks;
			done := 0;
			SpritesVisible := False;
			while done = 0 do
			begin
				{ Check for events }
				frames := frames + 1;
				while SDL_PollEvent(@event) <> 0 do
				begin
					case event.type_ of
					SDL_KEYDOWN:
						{ any keypress quits teh app }
						done := 1;
					SDL_QUITEV:
						done := 1;
					end;
				end;
				MoveSprites(screen, background);
			end;
			SDL_FreeSurface(sprite);
    	{ Print out some timing information }
			now := SDL_GetTicks;
			if now > AndThen then
			begin
				WriteLn(frames * 1000.0 / (now - AndThen) :3:2, ' frames per second');
			end;
		finally
			SDL_Quit
		end;
	end
end.

